@@ -1,8 +1,35 @@
 # -*- coding: utf-8 -*-
 
+from __future__ import absolute_import, division, print_function
+
+__copyright__ = """
+Copyright (C) 2009-2017 Andreas Kloeckner
+Copyright (C) 2014-2017 Aaron Meurer
+"""
+
+__license__ = """
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
+"""
+
+
 # {{{ constants and imports
 
-from __future__ import absolute_import, division, print_function
 import urwid
 
 try:
@@ -19,7 +46,7 @@ else:
     ELLIPSIS = unicode('…', 'utf-8')  # noqa: F821
 
 from pudb.debugger import CONFIG
-
+from pudb.ui_tools import text_width
 # }}}
 
 
@@ -110,7 +137,7 @@ class VariableWidget(urwid.FlowWidget):
         firstline = self.prefix + alltext[:maxcol]
         if not alltext[maxcol:]:
             return [firstline]
-        fulllines, rest = divmod(len(alltext) - maxcol, maxcol - 2)
+        fulllines, rest = divmod(text_width(alltext) - maxcol, maxcol - 2)
         restlines = [alltext[(maxcol - 2)*i + maxcol:(maxcol - 2)*i + 2*maxcol - 2]
             for i in xrange(fulllines + bool(rest))]
         return [firstline] + ["  " + self.prefix + i for i in restlines]
@@ -121,7 +148,7 @@ class VariableWidget(urwid.FlowWidget):
 
         if (self.value_str is not None
                 and self.var_label is not None
-                and len(self.prefix) + len(self.var_label) > self.SIZE_LIMIT):
+                and len(self.prefix) + text_width(self.var_label) > self.SIZE_LIMIT):
             return 2
         else:
             return 1
@@ -140,15 +167,15 @@ class VariableWidget(urwid.FlowWidget):
         if self.wrap:
             text = self._get_text(size)
 
-            extralabel_full, extralabel_rem = divmod(len(var_label[maxcol:]), maxcol)
-            totallen = sum([len(i) for i in text])
+            extralabel_full, extralabel_rem = divmod(text_width(var_label[maxcol:]), maxcol)
+            totallen = sum([text_width(i) for i in text])
             labellen = (
                     len(self.prefix)  # Padding of first line
 
                     + (len(self.prefix) + 2)  # Padding of subsequent lines
                     * (extralabel_full + bool(extralabel_rem))
 
-                    + len(var_label)
+                    + text_width(var_label)
 
                     + 2  # for ": "
                     )
@@ -165,31 +192,31 @@ class VariableWidget(urwid.FlowWidget):
 
         if self.value_str is not None:
             if self.var_label is not None:
-                if len(self.prefix) + len(self.var_label) > self.SIZE_LIMIT:
+                if len(self.prefix) + text_width(self.var_label) > self.SIZE_LIMIT:
                     # label too long? generate separate value line
                     text = [self.prefix + self.var_label,
                             self.prefix+"  " + self.value_str]
 
-                    attr = [[(apfx+"label", len(self.prefix)+len(self.var_label))],
-                            [(apfx+"value", len(self.prefix)+2+len(self.value_str))]]
+                    attr = [[(apfx+"label", len(self.prefix)+text_width(self.var_label))],
+                            [(apfx+"value", len(self.prefix)+2+text_width(self.value_str))]]
                 else:
                     text = [self.prefix + self.var_label + ": " + self.value_str]
 
                     attr = [[
-                            (apfx+"label", len(self.prefix)+len(self.var_label)+2),
-                            (apfx+"value", len(self.value_str)),
+                            (apfx+"label", len(self.prefix)+text_width(self.var_label)+2),
+                            (apfx+"value", text_width(self.value_str)),
                             ]]
             else:
                 text = [self.prefix + self.value_str]
 
                 attr = [[
                         (apfx+"label", len(self.prefix)),
-                        (apfx+"value", len(self.value_str)),
+                        (apfx+"value", text_width(self.value_str)),
                         ]]
         else:
             text = [self.prefix + self.var_label]
 
-            attr = [[(apfx+"label", len(self.prefix) + len(self.var_label)), ]]
+            attr = [[(apfx+"label", len(self.prefix) + text_width(self.var_label)), ]]
 
         # Ellipses to show text was cut off
         #encoding = urwid.util.detected_encoding
@@ -208,7 +235,7 @@ class VariableWidget(urwid.FlowWidget):
                     # + unicode(u'…')) + unicode(text[i][maxcol-2:])
         else:
             for i in xrange(len(text)):
-                if len(text[i]) > maxcol:
+                if text_width(text[i]) > maxcol:
                     text[i] = text[i][:maxcol-3] + "..."
 
         return make_canvas(text, attr, maxcol, apfx+"value")
@@ -232,7 +259,7 @@ def type_stringifier(value):
         except Exception:
             pass
 
-    elif hasattr(value, "safely_stringify_for_pudb"):
+    elif hasattr(type(value), "safely_stringify_for_pudb"):
         try:
             # (E.g.) Mock objects will pretend to have this
             # and return nonsense.
@@ -243,6 +270,9 @@ def type_stringifier(value):
             if isinstance(result, string_types):
                 return result
 
+    elif type(value) in [set, frozenset, list, tuple, dict]:
+        return "%s (%s)" % (type(value).__name__, len(value))
+
     return type(value).__name__
 
 
@@ -258,7 +288,7 @@ def get_stringifier(iinfo):
             if not custom_stringifier_dict:  # Only execfile once
                 from os.path import expanduser
                 execfile(expanduser(iinfo.display_type), custom_stringifier_dict)
-        except:
+        except Exception:
             print("Error when importing custom stringifier:")
             from traceback import print_exc
             print_exc()
@@ -347,23 +377,23 @@ class ValueWalker:
                     key_it = value.keys()
                 else:
                     key_it = value.iterkeys()
-            except:
+            except Exception:
                 pass
 
             if key_it is None:
                 try:
-                    l = len(value)
-                except:
+                    len_value = len(value)
+                except Exception:
                     pass
                 else:
                     try:
                         value[0]
                     except IndexError:
                         key_it = []
-                    except:
+                    except Exception:
                         pass
                     else:
-                        key_it = xrange(l)
+                        key_it = xrange(len_value)
 
             if key_it is not None:
                 cnt = 0
@@ -388,7 +418,7 @@ class ValueWalker:
 
             try:
                 key_its.append(dir(value))
-            except:
+            except Exception:
                 pass
 
             keys = [key
@@ -413,7 +443,7 @@ class ValueWalker:
                     if callable(attr_value) and not iinfo.show_methods:
                         cnt_omitted_methods += 1
                         continue
-                except:
+                except Exception:
                     attr_value = WatchEvalError()
 
                 self.walk_value(prefix+self.PREFIX,
@@ -512,7 +542,7 @@ def make_var_view(frame_var_info, locals, globals):
     for watch_expr in frame_var_info.watches:
         try:
             value = eval(watch_expr.expression, globals, locals)
-        except:
+        except Exception:
             value = WatchEvalError()
 
         WatchValueWalker(frame_var_info, watch_widget_list, watch_expr) \
@@ -523,7 +553,7 @@ def make_var_view(frame_var_info, locals, globals):
                 attr_prefix="return")
 
     for var in vars:
-        if not var[0] in "_.":
+        if not (var.startswith('__') and var.endswith('__')):
             tmv_walker.walk_value("", var, locals[var])
 
     result = tmv_walker.main_widget_list
